@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,7 +14,7 @@
 #include "ui_csvatlaswindow.h"
 
 class CSVAtlas;
-class XAbstractMessageHandler;
+class YAbstractMessageHandler;
 
 class CSVAtlasWindow : public QMainWindow, public Ui::CSVAtlasWindow
 {
@@ -32,19 +32,23 @@ class CSVAtlasWindow : public QMainWindow, public Ui::CSVAtlasWindow
     virtual void        filePrint();
     virtual void        fileSave();
     virtual void        fileSaveAs();
+    virtual void        dbOpen(QString filename = QString());
+    virtual void        dbSave();
+    virtual void        dbDelete();
     virtual CSVAtlas   *getAtlas();
+    virtual bool        hasKey();
     virtual void        helpAbout();
     virtual void        helpContents();
     virtual void        helpIndex();
     virtual QString     map()                         const;
-    virtual XAbstractMessageHandler *messageHandler() const;
+    virtual YAbstractMessageHandler *messageHandler() const;
     virtual void        sAddMap();
     virtual void        sDeleteMap();
     virtual void        sMapChanged( int );
     virtual void        sRenameMap();
     virtual void        setDir(QString dirname);
     virtual bool        setMap(const QString mapname);
-    virtual void        setMessageHandler(XAbstractMessageHandler *handler);
+    virtual void        setMessageHandler(YAbstractMessageHandler *handler);
 
   signals:
     void delimiterChanged(QString);
@@ -56,7 +60,7 @@ class CSVAtlasWindow : public QMainWindow, public Ui::CSVAtlasWindow
     CSVAtlas                 *_atlas;
     QString                  _currentDir;
     QString                  _filename;
-    XAbstractMessageHandler *_msghandler;
+    YAbstractMessageHandler *_msghandler;
     QString                  _selectedMap;
 };
 
