@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -326,12 +326,12 @@ bool CSVData::load(QString filename, QWidget *parent)
   return result;
 }
 
-XAbstractMessageHandler *CSVData::messageHandler() const
+YAbstractMessageHandler *CSVData::messageHandler() const
 {
   return _msghandler;
 }
 
-void CSVData::setMessageHandler(XAbstractMessageHandler *handler)
+void CSVData::setMessageHandler(YAbstractMessageHandler *handler)
 {
   _msghandler = handler;
 }
