@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,9 +11,9 @@
 #ifndef INTERACTIVEMESSAGEHANDLER_H
 #define INTERACTIVEMESSAGEHANDLER_H
 
-#include "xabstractmessagehandler.h"
+#include "yabstractmessagehandler.h"
 
-class InteractiveMessageHandler : public XAbstractMessageHandler
+class InteractiveMessageHandler : public YAbstractMessageHandler
 {
   Q_OBJECT
     
