@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -32,6 +32,7 @@ class RowController : public QObject
     void setAltColumn(QSpinBox*);
     void setAltIfNull(QComboBox*);
     void setAltValue(QTableWidgetItem*);
+    void setFileType(QComboBox*);
 
   public slots:
     void finishSetup();
@@ -47,6 +48,7 @@ class RowController : public QObject
     QSpinBox         *_altColumn; // 7
     QComboBox        *_altIfNull; // 8
     QTableWidgetItem *_altValue;  // 9
+    QComboBox        *_fileType;  // 10
 };
 
 #endif
